@@ -30,7 +30,8 @@ if [[ ${UPSTREAM_DEV} ]]; then
         git+https://github.com/dask/zict \
         git+https://github.com/dask/distributed \
         git+https://github.com/dask/fastparquet \
-        git+https://github.com/zarr-developers/zarr-python
+        git+https://github.com/zarr-developers/zarr-python \
+        git+https://github.com/PyTables/PyTables  # numpy 2 support
 
     # FIXME https://github.com/mamba-org/mamba/issues/412
     # mamba uninstall --force ...
@@ -53,7 +54,7 @@ if [[ ${UPSTREAM_DEV} ]]; then
 
     # FIXME https://github.com/mamba-org/mamba/issues/412
     # mamba uninstall --force ...
-    conda uninstall --force crick tiledb tiledb-py
+    # conda uninstall --force crick tiledb tiledb-py
 
 
 fi
