@@ -1,8 +1,11 @@
+from __future__ import annotations
+
 import math
 
 import numpy as np
 
 from dask.array import chunk
+from dask.array.core import Array
 from dask.array.dispatch import (
     concatenate_lookup,
     divide_lookup,
@@ -12,6 +15,8 @@ from dask.array.dispatch import (
     numel_lookup,
     percentile_lookup,
     tensordot_lookup,
+    to_cupy_dispatch,
+    to_numpy_dispatch,
 )
 from dask.array.numpy_compat import divide as np_divide
 from dask.array.numpy_compat import ma_divide
@@ -120,17 +125,24 @@ def _tensordot(a, b, axes=2):
 @concatenate_lookup.register_lazy("cupy")
 @nannumel_lookup.register_lazy("cupy")
 @numel_lookup.register_lazy("cupy")
+@to_numpy_dispatch.register_lazy("cupy")
 def register_cupy():
     import cupy
 
-    from dask.array.dispatch import percentile_lookup
-
     concatenate_lookup.register(cupy.ndarray, cupy.concatenate)
     tensordot_lookup.register(cupy.ndarray, cupy.tensordot)
     percentile_lookup.register(cupy.ndarray, percentile)
     numel_lookup.register(cupy.ndarray, _numel_arraylike)
     nannumel_lookup.register(cupy.ndarray, _nannumel)
 
+    @to_numpy_dispatch.register(cupy.ndarray)
+    def cupy_to_numpy(data, **kwargs):
+        return cupy.asnumpy(data, **kwargs)
+
+    @to_cupy_dispatch.register(np.ndarray)
+    def numpy_to_cupy(data, **kwargs):
+        return cupy.asarray(data, **kwargs)
+
     @einsum_lookup.register(cupy.ndarray)
     def _cupy_einsum(*args, **kwargs):
         # NB: cupy does not accept `order` or `casting` kwargs - ignore
@@ -142,7 +154,6 @@ def register_cupy():
 @tensordot_lookup.register_lazy("cupyx")
 @concatenate_lookup.register_lazy("cupyx")
 def register_cupyx():
-
     from cupyx.scipy.sparse import spmatrix
 
     try:
@@ -316,6 +327,11 @@ class ArrayBackendEntrypoint(DaskBackendEntrypoint):
         """
         raise NotImplementedError
 
+    @property
+    def default_bit_generator(self):
+        """Return the default BitGenerator type"""
+        raise NotImplementedError
+
     @staticmethod
     def ones(shape, *, dtype=None, meta=None, **kwargs):
         """Create an array of ones
@@ -361,11 +377,31 @@ class ArrayBackendEntrypoint(DaskBackendEntrypoint):
         raise NotImplementedError
 
 
+@to_numpy_dispatch.register(np.ndarray)
+def to_numpy_dispatch_from_numpy(data, **kwargs):
+    return data
+
+
 class NumpyBackendEntrypoint(ArrayBackendEntrypoint):
+    @classmethod
+    def to_backend_dispatch(cls):
+        return to_numpy_dispatch
+
+    @classmethod
+    def to_backend(cls, data: Array, **kwargs):
+        if isinstance(data._meta, np.ndarray):
+            # Already a numpy-backed collection
+            return data
+        return data.map_blocks(cls.to_backend_dispatch(), **kwargs)
+
     @property
     def RandomState(self):
         return np.random.RandomState
 
+    @property
+    def default_bit_generator(self):
+        return np.random.PCG64
+
 
 array_creation_dispatch = CreationDispatch(
     module_name="array",
