@@ -9,13 +9,8 @@ from numbers import Integral, Real
 import numpy as np
 from tlz import concat, interleave, sliding_window
 
-from ..base import is_dask_collection, tokenize
-from ..core import flatten
-from ..delayed import Delayed, unpack_collections
-from ..highlevelgraph import HighLevelGraph
-from ..utils import apply, derived_from, funcname, is_arraylike, is_cupy_type
-from . import chunk
-from .core import (
+from dask.array import chunk
+from dask.array.core import (
     Array,
     asanyarray,
     asarray,
@@ -25,19 +20,30 @@ from .core import (
     broadcast_to,
     concatenate,
     elemwise,
+    from_array,
     implements,
     is_scalar_for_elemwise,
     map_blocks,
     stack,
     tensordot_lookup,
 )
-from .creation import arange, diag, empty, indices, tri
-from .einsumfuncs import einsum  # noqa
-from .numpy_compat import _numpy_120
-from .reductions import reduction
-from .ufunc import multiply, sqrt
-from .utils import array_safe, asarray_safe, meta_from_array, safe_wraps, validate_axis
-from .wrap import ones
+from dask.array.creation import arange, diag, empty, indices, tri
+from dask.array.einsumfuncs import einsum  # noqa
+from dask.array.reductions import reduction
+from dask.array.ufunc import multiply, sqrt
+from dask.array.utils import (
+    array_safe,
+    asarray_safe,
+    meta_from_array,
+    safe_wraps,
+    validate_axis,
+)
+from dask.array.wrap import ones
+from dask.base import is_dask_collection, tokenize
+from dask.core import flatten
+from dask.delayed import Delayed, unpack_collections
+from dask.highlevelgraph import HighLevelGraph
+from dask.utils import apply, derived_from, funcname, is_arraylike, is_cupy_type
 
 # save built-in for histogram functions which use range as a kwarg.
 _range = range
@@ -45,8 +51,6 @@ _range = range
 
 @derived_from(np)
 def array(x, dtype=None, ndmin=None, *, like=None):
-    if not _numpy_120 and like is not None:
-        raise RuntimeError("The use of ``like`` required NumPy >= 1.20")
     x = asarray(x, like=like)
     while ndmin is not None and x.ndim < ndmin:
         x = x[None, :]
@@ -259,28 +263,40 @@ def rot90(m, k=1, axes=(0, 1)):
         return flip(transpose(m, axes_list), axes[1])
 
 
-def _tensordot(a, b, axes):
+def _tensordot(a, b, axes, is_sparse):
     x = max([a, b], key=lambda x: x.__array_priority__)
     tensordot = tensordot_lookup.dispatch(type(x))
     x = tensordot(a, b, axes=axes)
-
-    if len(axes[0]) != 1:
+    if is_sparse and len(axes[0]) == 1:
+        return x
+    else:
         ind = [slice(None, None)] * x.ndim
         for a in sorted(axes[0]):
             ind.insert(a, None)
         x = x[tuple(ind)]
+        return x
 
-    return x
+
+def _tensordot_is_sparse(x):
+    is_sparse = "sparse" in str(type(x._meta))
+    if is_sparse:
+        # exclude pydata sparse arrays, no workaround required for these in tensordot
+        is_sparse = "sparse._coo.core.COO" not in str(type(x._meta))
+    return is_sparse
 
 
 @derived_from(np)
 def tensordot(lhs, rhs, axes=2):
+    if not isinstance(lhs, Array):
+        lhs = from_array(lhs)
+    if not isinstance(rhs, Array):
+        rhs = from_array(rhs)
+
     if isinstance(axes, Iterable):
         left_axes, right_axes = axes
     else:
         left_axes = tuple(range(lhs.ndim - axes, lhs.ndim))
         right_axes = tuple(range(0, axes))
-
     if isinstance(left_axes, Integral):
         left_axes = (left_axes,)
     if isinstance(right_axes, Integral):
@@ -289,23 +305,23 @@ def tensordot(lhs, rhs, axes=2):
         left_axes = tuple(left_axes)
     if isinstance(right_axes, list):
         right_axes = tuple(right_axes)
-    if len(left_axes) == 1:
+    is_sparse = _tensordot_is_sparse(lhs) or _tensordot_is_sparse(rhs)
+    if is_sparse and len(left_axes) == 1:
         concatenate = True
     else:
         concatenate = False
-
     dt = np.promote_types(lhs.dtype, rhs.dtype)
-
     left_index = list(range(lhs.ndim))
     right_index = list(range(lhs.ndim, lhs.ndim + rhs.ndim))
     out_index = left_index + right_index
-
+    adjust_chunks = {}
     for l, r in zip(left_axes, right_axes):
         out_index.remove(right_index[r])
         right_index[r] = left_index[l]
         if concatenate:
             out_index.remove(left_index[l])
-
+        else:
+            adjust_chunks[left_index[l]] = lambda c: 1
     intermediate = blockwise(
         _tensordot,
         out_index,
@@ -315,16 +331,17 @@ def tensordot(lhs, rhs, axes=2):
         right_index,
         dtype=dt,
         concatenate=concatenate,
+        adjust_chunks=adjust_chunks,
         axes=(left_axes, right_axes),
+        is_sparse=is_sparse,
     )
-
     if concatenate:
         return intermediate
     else:
         return intermediate.sum(axis=left_axes)
 
 
-@derived_from(np)
+@derived_from(np, ua_args=["out"])
 def dot(a, b):
     return tensordot(a, b, axes=((a.ndim - 1,), (b.ndim - 2,)))
 
@@ -590,7 +607,7 @@ def diff(a, n=1, axis=-1, prepend=None, append=None):
     sl_2 = tuple(sl_2)
 
     r = a
-    for i in range(n):
+    for _ in range(n):
         r = r[sl_1] - r[sl_2]
 
     return r
@@ -664,7 +681,7 @@ def gradient(f, *varargs, axis=None, **kwargs):
             "Spacing must either be a single scalar, or a scalar / 1d-array per axis"
         )
 
-    if issubclass(f.dtype.type, (np.bool8, Integral)):
+    if issubclass(f.dtype.type, (np.bool_, Integral)):
         f = f.astype(float)
     elif issubclass(f.dtype.type, Real) and f.dtype.itemsize < 4:
         f = f.astype(float)
@@ -751,7 +768,7 @@ def bincount(x, weights=None, minlength=0, split_every=None):
         *chunked_counts.chunks[1:],
     )
 
-    from .reductions import _tree_reduce
+    from dask.array.reductions import _tree_reduce
 
     output = _tree_reduce(
         chunked_counts,
@@ -1549,7 +1566,7 @@ def corrcoef(x, y=None, rowvar=1):
     return (c / sqr_d) / sqr_d.T
 
 
-@implements(np.round, np.round_)
+@implements(np.round)
 @derived_from(np)
 def round(a, decimals=0):
     return a.map_blocks(np.round, decimals=decimals, dtype=a.dtype)
@@ -1794,7 +1811,7 @@ def unique(ar, return_index=False, return_inverse=False, return_counts=False):
 
 
 def _isin_kernel(element, test_elements, assume_unique=False):
-    values = np.in1d(element.ravel(), test_elements, assume_unique=assume_unique)
+    values = np.isin(element.ravel(), test_elements, assume_unique=assume_unique)
     return values.reshape(element.shape + (1,) * test_elements.ndim)
 
 
@@ -1850,8 +1867,8 @@ def roll(array, shift, axis=None):
         raise ValueError("Must have the same number of shifts as axes.")
 
     for i, s in zip(axis, shift):
-        s = -s
-        s %= result.shape[i]
+        shape = result.shape[i]
+        s = 0 if shape == 0 else -s % shape
 
         sl1 = result.ndim * [slice(None)]
         sl2 = result.ndim * [slice(None)]
@@ -1865,6 +1882,8 @@ def roll(array, shift, axis=None):
         result = concatenate([result[sl1], result[sl2]], axis=i)
 
     result = result.reshape(array.shape)
+    # Ensure that the output is always a new array object
+    result = result.copy() if result is array else result
 
     return result
 
@@ -1912,6 +1931,13 @@ def squeeze(a, axis=None):
 
     sl = tuple(0 if i in axis else slice(None) for i, s in enumerate(a.shape))
 
+    # Return 0d Dask Array if all axes are squeezed,
+    # to be consistent with NumPy. Ref: https://github.com/dask/dask/issues/9183#issuecomment-1155626619
+    if all(s == 0 for s in sl) and all(s == 1 for s in a.shape):
+        return a.map_blocks(
+            np.squeeze, meta=a._meta, drop_axis=tuple(range(len(a.shape)))
+        )
+
     a = a[sl]
 
     return a
@@ -1919,7 +1945,6 @@ def squeeze(a, axis=None):
 
 @derived_from(np)
 def compress(condition, a, axis=None):
-
     if not is_arraylike(condition):
         # Allow `condition` to be anything array-like, otherwise ensure `condition`
         # is a numpy array.
@@ -2292,8 +2317,11 @@ def coarsen(reduction, x, axes, trim_excess=False, **kwargs):
         + key[1:]: (apply, chunk.coarsen, [reduction, key, axes, trim_excess], kwargs)
         for key in flatten(x.__dask_keys__())
     }
+
+    coarsen_dim = lambda dim, ax: int(dim // axes.get(ax, 1))
     chunks = tuple(
-        tuple(int(bd // axes.get(i, 1)) for bd in bds) for i, bds in enumerate(x.chunks)
+        tuple(coarsen_dim(bd, i) for bd in bds if coarsen_dim(bd, i) > 0)
+        for i, bds in enumerate(x.chunks)
     )
 
     meta = reduction(np.empty((1,) * x.ndim, dtype=x.dtype), **kwargs)
@@ -2406,7 +2434,9 @@ def append(arr, values, axis=None):
     return concatenate((arr, values), axis=axis)
 
 
-def _average(a, axis=None, weights=None, returned=False, is_masked=False):
+def _average(
+    a, axis=None, weights=None, returned=False, is_masked=False, keepdims=False
+):
     # This was minimally modified from numpy.average
     # See numpy license at https://github.com/numpy/numpy/blob/master/LICENSE.txt
     # or NUMPY_LICENSE.txt within this directory
@@ -2414,7 +2444,7 @@ def _average(a, axis=None, weights=None, returned=False, is_masked=False):
     a = asanyarray(a)
 
     if weights is None:
-        avg = a.mean(axis)
+        avg = a.mean(axis, keepdims=keepdims)
         scl = avg.dtype.type(a.size / avg.size)
     else:
         wgt = asanyarray(weights)
@@ -2443,11 +2473,11 @@ def _average(a, axis=None, weights=None, returned=False, is_masked=False):
             wgt = broadcast_to(wgt, (a.ndim - 1) * (1,) + wgt.shape)
             wgt = wgt.swapaxes(-1, axis)
         if is_masked:
-            from .ma import getmaskarray
+            from dask.array.ma import getmaskarray
 
             wgt = wgt * (~getmaskarray(a))
-        scl = wgt.sum(axis=axis, dtype=result_dtype)
-        avg = multiply(a, wgt, dtype=result_dtype).sum(axis) / scl
+        scl = wgt.sum(axis=axis, dtype=result_dtype, keepdims=keepdims)
+        avg = multiply(a, wgt, dtype=result_dtype).sum(axis, keepdims=keepdims) / scl
 
     if returned:
         if scl.shape != avg.shape:
@@ -2458,8 +2488,8 @@ def _average(a, axis=None, weights=None, returned=False, is_masked=False):
 
 
 @derived_from(np)
-def average(a, axis=None, weights=None, returned=False):
-    return _average(a, axis, weights, returned, is_masked=False)
+def average(a, axis=None, weights=None, returned=False, keepdims=False):
+    return _average(a, axis, weights, returned, is_masked=False, keepdims=keepdims)
 
 
 @derived_from(np)
@@ -2470,7 +2500,7 @@ def tril(m, k=0):
         k=k,
         dtype=bool,
         chunks=m.chunks[-2:],
-        like=meta_from_array(m) if _numpy_120 else None,
+        like=meta_from_array(m),
     )
 
     return where(mask, m, np.zeros_like(m, shape=(1,)))
@@ -2484,7 +2514,7 @@ def triu(m, k=0):
         k=k - 1,
         dtype=bool,
         chunks=m.chunks[-2:],
-        like=meta_from_array(m) if _numpy_120 else None,
+        like=meta_from_array(m),
     )
 
     return where(mask, np.zeros_like(m, shape=(1,)), m)
