@@ -1,7 +1,10 @@
+from __future__ import annotations
+
 import gc
 import math
 import os
 import random
+import warnings
 import weakref
 from bz2 import BZ2File
 from collections.abc import Iterator
@@ -29,11 +32,13 @@ from dask.bag.core import (
     total_mem_usage,
 )
 from dask.bag.utils import assert_eq
+from dask.blockwise import Blockwise
 from dask.delayed import Delayed
+from dask.typing import Graph
 from dask.utils import filetexts, tmpdir, tmpfile
-from dask.utils_test import add, hlg_layer_topological, inc
+from dask.utils_test import add, hlg_layer, hlg_layer_topological, inc
 
-dsk = {("x", 0): (range, 5), ("x", 1): (range, 5), ("x", 2): (range, 5)}
+dsk: Graph = {("x", 0): (range, 5), ("x", 1): (range, 5), ("x", 2): (range, 5)}
 
 L = list(range(5)) * 3
 
@@ -471,6 +476,16 @@ def test_map_partitions_args_kwargs():
     assert_eq(dx.map_partitions(maximum, dy_mean), sol)
 
 
+def test_map_partitions_blockwise():
+    # Check that the `token` argument works,
+    # and that `map_partitions`` is using `Blockwise`.
+    layer = hlg_layer(
+        b.map_partitions(lambda x: x, token="test-string").dask, "test-string"
+    )
+    assert layer
+    assert isinstance(layer, Blockwise)
+
+
 def test_random_sample_size():
     """
     Number of randomly sampled elements are in the expected range.
@@ -605,13 +620,10 @@ def test_take_npartitions_warn():
         with pytest.warns(UserWarning):
             b.take(7)
 
-        with pytest.warns(None) as rec:
+        with warnings.catch_warnings(record=True) as record:
             b.take(7, npartitions=2)
-        assert len(rec) == 0
-
-        with pytest.warns(None) as rec:
             b.take(7, warn=False)
-        assert len(rec) == 0
+        assert not record
 
 
 def test_map_is_lazy():
@@ -962,8 +974,7 @@ def test_to_textfiles_name_function_warn():
     ]
     a = db.from_sequence(seq, npartitions=16)
     with tmpdir() as dn:
-        with pytest.warns(None):
-            a.to_textfiles(dn, name_function=str)
+        a.to_textfiles(dn, name_function=str)
 
 
 def test_to_textfiles_encoding():
@@ -1182,7 +1193,8 @@ def test_zip(npartitions, hi=1000):
     evens = db.from_sequence(range(0, hi, 2), npartitions=npartitions)
     odds = db.from_sequence(range(1, hi, 2), npartitions=npartitions)
     pairs = db.zip(evens, odds)
-    assert pairs.npartitions == npartitions
+    assert pairs.npartitions == evens.npartitions
+    assert pairs.npartitions == odds.npartitions
     assert list(pairs) == list(zip(range(0, hi, 2), range(1, hi, 2)))
 
 
@@ -1391,7 +1403,7 @@ def test_reduction_with_sparse_matrices():
 
 
 def test_empty():
-    list(db.from_sequence([])) == []
+    assert list(db.from_sequence([])) == []
 
 
 def test_bag_picklable():
@@ -1614,6 +1626,7 @@ def test_dask_layers_to_delayed(optimize):
 def test_to_dataframe_optimize_graph():
     pytest.importorskip("dask.dataframe")
     from dask.dataframe.utils import assert_eq as assert_eq_df
+    from dask.dataframe.utils import pyarrow_strings_enabled
 
     x = db.from_sequence(
         [{"name": "test1", "v1": 1}, {"name": "test2", "v1": 2}], npartitions=2
@@ -1632,16 +1645,49 @@ def test_to_dataframe_optimize_graph():
     d = y.to_dataframe()
 
     # All the `map` tasks have been fused
-    assert len(d.dask) < len(y.dask)
+    assert len(d.dask) < len(y.dask) + d.npartitions * int(pyarrow_strings_enabled())
 
     # no optimizations
     d2 = y.to_dataframe(optimize_graph=False)
 
     # Graph hasn't been fused. It contains all the original tasks,
     # plus one extra layer converting to DataFrame
-    assert len(d2.dask) == len(y.dask) + d.npartitions
+    assert len(d2.dask.keys() - y.dask.keys()) == d.npartitions * (
+        1 + int(pyarrow_strings_enabled())
+    )
 
     # Annotations are still there
     assert hlg_layer_topological(d2.dask, 1).annotations == {"foo": True}
 
     assert_eq_df(d, d2)
+
+
+@pytest.mark.parametrize("nworkers", [100, 250, 500, 1000])
+def test_default_partitioning_worker_saturation(nworkers):
+    # Ensure that Dask Bag can saturate any number of workers with concurrent tasks.
+    # The default partitioning scheme partitions items to keep the task to item ratio sensible
+    # but it should always be possible to saturate any number of workers given enough items in the bag.
+    ntasks = 0
+    nitems = 1
+    while ntasks < nworkers:
+        ntasks = len(db.from_sequence(range(nitems)).dask)
+        nitems += math.floor(max(1, nworkers / 10))
+        assert nitems < 20_000
+
+
+@pytest.mark.parametrize("nworkers", [100, 250, 500, 1000])
+def test_npartitions_saturation(nworkers):
+    # If npartitions is set the bag should always contain at least that number of tasks
+    for nitems in range(nworkers, 10 * nworkers, max(1, math.floor(nworkers / 10))):
+        assert (
+            len(db.from_sequence(range(nitems), npartitions=nworkers).dask) >= nworkers
+        )
+
+
+def test_map_total_mem_usage():
+    """https://github.com/dask/dask/issues/10338"""
+    b = db.from_sequence(range(1, 100), npartitions=3)
+    total_mem_b = sum(b.map_partitions(total_mem_usage).compute())
+    c = b.map(lambda x: x)
+    total_mem_c = sum(c.map_partitions(total_mem_usage).compute())
+    assert total_mem_b == total_mem_c
