@@ -1,37 +1,50 @@
 from __future__ import annotations
 
+import dataclasses
 import datetime
+import hashlib
 import inspect
 import os
+import pathlib
 import pickle
 import threading
 import uuid
+import warnings
 from collections import OrderedDict
-from collections.abc import Callable, Iterator, Mapping
+from collections.abc import Callable, Hashable, Iterator, Mapping
 from concurrent.futures import Executor
 from contextlib import contextmanager
-from dataclasses import fields, is_dataclass
+from contextvars import ContextVar
+from enum import Enum
 from functools import partial
-from hashlib import md5
 from numbers import Integral, Number
 from operator import getitem
+from typing import TYPE_CHECKING, Any, Literal, Protocol, TypeVar
 
-from packaging.version import parse as parse_version
 from tlz import curry, groupby, identity, merge
 from tlz.functoolz import Compose
 
-from . import config, local, threaded
-from .context import thread_state
-from .core import flatten
-from .core import get as simple_get
-from .core import literal, quote
-from .hashing import hash_buffer_hex
-from .system import CPU_COUNT
-from .utils import Dispatch, apply, ensure_dict, key_split
+from dask import config, local
+from dask._compatibility import EMSCRIPTEN
+from dask.core import flatten
+from dask.core import get as simple_get
+from dask.core import literal, quote
+from dask.hashing import hash_buffer_hex
+from dask.system import CPU_COUNT
+from dask.typing import Key, SchedulerGetCallable
+from dask.utils import (
+    Dispatch,
+    apply,
+    ensure_dict,
+    is_namedtuple_instance,
+    key_split,
+    shorten_traceback,
+)
 
 __all__ = (
     "DaskMethodsMixin",
     "annotate",
+    "get_annotations",
     "is_dask_collection",
     "compute",
     "persist",
@@ -45,9 +58,28 @@ __all__ = (
     "clone_key",
 )
 
+if TYPE_CHECKING:
+    from _typeshed import ReadableBuffer
+
+_annotations: ContextVar[dict[str, Any]] = ContextVar("annotations", default={})
+
+
+def get_annotations() -> dict[str, Any]:
+    """Get current annotations.
+
+    Returns
+    -------
+    Dict of all current annotations
+
+    See Also
+    --------
+    annotate
+    """
+    return _annotations.get()
+
 
 @contextmanager
-def annotate(**annotations):
+def annotate(**annotations: Any) -> Iterator[None]:
     """Context Manager for setting HighLevelGraph Layer annotations.
 
     Annotations are metadata or soft constraints associated with
@@ -89,6 +121,10 @@ def annotate(**annotations):
     ...     with dask.annotate(retries=3):
     ...         A = da.ones((1000, 1000))
     ...     B = A + 1
+
+    See Also
+    --------
+    get_annotations
     """
 
     # Sanity check annotations used in place of
@@ -146,18 +182,34 @@ def annotate(**annotations):
             % annotations["allow_other_workers"]
         )
 
-    prev_annotations = config.get("annotations", {})
-    new_annotations = {
-        **prev_annotations,
-        **{f"annotations.{k}": v for k, v in annotations.items()},
-    }
-
-    with config.set(new_annotations):
+    token = _annotations.set(merge(_annotations.get(), annotations))
+    try:
         yield
+    finally:
+        _annotations.reset(token)
 
 
-def is_dask_collection(x):
-    """Returns ``True`` if ``x`` is a dask collection"""
+def is_dask_collection(x) -> bool:
+    """Returns ``True`` if ``x`` is a dask collection.
+
+    Parameters
+    ----------
+    x : Any
+        Object to test.
+
+    Returns
+    -------
+    result : bool
+        ``True`` if `x` is a Dask collection.
+
+    Notes
+    -----
+    The DaskCollection typing.Protocol implementation defines a Dask
+    collection as a class that returns a Mapping from the
+    ``__dask_graph__`` method. This helper function existed before the
+    implementation of the protocol.
+
+    """
     try:
         return x.__dask_graph__() is not None
     except (AttributeError, TypeError):
@@ -204,7 +256,7 @@ class DaskMethodsMixin:
 
         See Also
         --------
-        dask.base.visualize
+        dask.visualize
         dask.dot.dot_graph
 
         Notes
@@ -258,7 +310,7 @@ class DaskMethodsMixin:
 
         See Also
         --------
-        dask.base.persist
+        dask.persist
         """
         (result,) = persist(self, traverse=False, **kwargs)
         return result
@@ -285,7 +337,7 @@ class DaskMethodsMixin:
 
         See Also
         --------
-        dask.base.compute
+        dask.compute
         """
         (result,) = compute(self, traverse=False, **kwargs)
         return result
@@ -329,7 +381,7 @@ def collections_to_dsk(collections, optimize_graph=True, optimizations=(), **kwa
     """
     Convert many collections into a single dask graph, after optimization
     """
-    from .highlevelgraph import HighLevelGraph
+    from dask.highlevelgraph import HighLevelGraph
 
     optimizations = tuple(optimizations) + tuple(config.get("optimizations", ()))
 
@@ -360,7 +412,7 @@ def collections_to_dsk(collections, optimize_graph=True, optimizations=(), **kwa
 def _extract_graph_and_keys(vals):
     """Given a list of dask vals, return a single graph and a list of keys such
     that ``get(dsk, keys)`` is equivalent to ``[v.compute() for v in vals]``."""
-    from .highlevelgraph import HighLevelGraph
+    from dask.highlevelgraph import HighLevelGraph
 
     graphs, keys = [], []
     for v in vals:
@@ -425,7 +477,7 @@ def unpack_collections(*args, traverse=True):
                 tsk = (typ, [_unpack(i) for i in expr])
             elif typ in (dict, OrderedDict):
                 tsk = (typ, [[_unpack(k), _unpack(v)] for k, v in expr.items()])
-            elif is_dataclass(expr) and not isinstance(expr, type):
+            elif dataclasses.is_dataclass(expr) and not isinstance(expr, type):
                 tsk = (
                     apply,
                     typ,
@@ -434,10 +486,12 @@ def unpack_collections(*args, traverse=True):
                         dict,
                         [
                             [f.name, _unpack(getattr(expr, f.name))]
-                            for f in fields(expr)
+                            for f in dataclasses.fields(expr)
                         ],
                     ),
                 )
+            elif is_namedtuple_instance(expr):
+                tsk = (typ, *[_unpack(i) for i in expr])
             else:
                 return expr
 
@@ -484,11 +538,11 @@ def optimize(*args, traverse=True, **kwargs):
 
     Examples
     --------
-    >>> import dask as d
+    >>> import dask
     >>> import dask.array as da
     >>> a = da.arange(10, chunks=2).sum()
     >>> b = da.arange(10, chunks=2).mean()
-    >>> a2, b2 = d.optimize(a, b)
+    >>> a2, b2 = dask.optimize(a, b)
 
     >>> a2.compute() == a.compute()
     True
@@ -541,16 +595,16 @@ def compute(
 
     Examples
     --------
-    >>> import dask as d
+    >>> import dask
     >>> import dask.array as da
     >>> a = da.arange(10, chunks=2).sum()
     >>> b = da.arange(10, chunks=2).mean()
-    >>> d.compute(a, b)
+    >>> dask.compute(a, b)
     (45, 4.5)
 
     By default, dask objects inside python collections will also be computed:
 
-    >>> d.compute({'a': a, 'b': b, 'c': 1})
+    >>> dask.compute({'a': a, 'b': b, 'c': 1})
     ({'a': 45, 'b': 4.5, 'c': 1},)
     """
 
@@ -570,12 +624,20 @@ def compute(
         keys.append(x.__dask_keys__())
         postcomputes.append(x.__dask_postcompute__())
 
-    results = schedule(dsk, keys, **kwargs)
+    with shorten_traceback():
+        results = schedule(dsk, keys, **kwargs)
+
     return repack([f(r, *a) for r, (f, a) in zip(results, postcomputes)])
 
 
 def visualize(
-    *args, filename="mydask", traverse=True, optimize_graph=False, maxval=None, **kwargs
+    *args,
+    filename="mydask",
+    traverse=True,
+    optimize_graph=False,
+    maxval=None,
+    engine: Literal["cytoscape", "ipycytoscape", "graphviz"] | None = None,
+    **kwargs,
 ):
     """
     Visualize several dask graphs simultaneously.
@@ -586,7 +648,8 @@ def visualize(
     Parameters
     ----------
     args : object
-        Any number of objects. If it is a dask object, its associated graph
+        Any number of objects. If it is a dask collection (for example, a
+        dask DataFrame, Array, Bag, or Delayed), its associated graph
         will be included in the output of visualize. By default, python builtin
         collections are also traversed to look for dask objects (for more
         information see the ``traverse`` keyword). Arguments lacking an
@@ -628,8 +691,12 @@ def visualize(
     verbose : bool, optional
         Whether to label output and input boxes even if the data aren't chunked.
         Beware: these labels can get very long. Default is False.
+    engine : {"graphviz", "ipycytoscape", "cytoscape"}, optional.
+        The visualization engine to use. If not provided, this checks the dask config
+        value "visualization.engine". If that is not set, it tries to import ``graphviz``
+        and ``ipycytoscape``, using the first one to succeed.
     **kwargs
-       Additional keyword arguments to forward to ``to_graphviz``.
+       Additional keyword arguments to forward to the visualization engine.
 
     Examples
     --------
@@ -651,13 +718,34 @@ def visualize(
 
     https://docs.dask.org/en/latest/optimize.html
     """
-    from dask.dot import dot_graph
-
     args, _ = unpack_collections(*args, traverse=traverse)
 
     dsk = dict(collections_to_dsk(args, optimize_graph=optimize_graph))
 
+    return visualize_dsk(
+        dsk=dsk,
+        filename=filename,
+        traverse=traverse,
+        optimize_graph=optimize_graph,
+        maxval=maxval,
+        engine=engine,
+        **kwargs,
+    )
+
+
+def visualize_dsk(
+    dsk,
+    filename="mydask",
+    traverse=True,
+    optimize_graph=False,
+    maxval=None,
+    o=None,
+    engine: Literal["cytoscape", "ipycytoscape", "graphviz"] | None = None,
+    limit=None,
+    **kwargs,
+):
     color = kwargs.get("color")
+    from dask.order import diagnostics, order
 
     if color in {
         "order",
@@ -671,16 +759,24 @@ def visualize(
         "memoryincreases",
         "memorydecreases",
         "memorypressure",
+        "critical",
+        "cpath",
     }:
         import matplotlib.pyplot as plt
 
-        from .order import diagnostics, order
+        if o is None:
+            o_stats = order(dsk, return_stats=True)
+            o = {k: v.priority for k, v in o_stats.items()}
+        elif isinstance(next(iter(o.values())), int):
+            o_stats = order(dsk, return_stats=True)
+        else:
+            o_stats = o
+            o = {k: v.priority for k, v in o.items()}
 
-        o = order(dsk)
         try:
             cmap = kwargs.pop("cmap")
         except KeyError:
-            cmap = plt.cm.RdBu
+            cmap = plt.cm.plasma
         if isinstance(cmap, str):
             import matplotlib.pyplot as plt
 
@@ -706,11 +802,15 @@ def visualize(
                     key: max(0, val.num_data_when_released - val.num_data_when_run)
                     for key, val in info.items()
                 }
-            else:  # memorydecreases
+            elif color.endswith("memorydecreases"):
                 values = {
                     key: max(0, val.num_data_when_run - val.num_data_when_released)
                     for key, val in info.items()
                 }
+            elif color.split("-")[-1] in {"critical", "cpath"}:
+                values = {key: val.critical_path for key, val in o_stats.items()}
+            else:
+                raise NotImplementedError(color)
 
             if color.startswith("order-"):
 
@@ -738,7 +838,36 @@ def visualize(
     elif color:
         raise NotImplementedError("Unknown value color=%s" % color)
 
-    return dot_graph(dsk, filename=filename, **kwargs)
+    # Determine which engine to dispatch to, first checking the kwarg, then config,
+    # then whichever of graphviz or ipycytoscape are installed, in that order.
+    engine = engine or config.get("visualization.engine", None)
+
+    if not engine:
+        try:
+            import graphviz  # noqa: F401
+
+            engine = "graphviz"
+        except ImportError:
+            try:
+                import ipycytoscape  # noqa: F401
+
+                engine = "cytoscape"
+            except ImportError:
+                pass
+    if engine == "graphviz":
+        from dask.dot import dot_graph
+
+        return dot_graph(dsk, filename=filename, **kwargs)
+    elif engine in ("cytoscape", "ipycytoscape"):
+        from dask.dot import cytoscape_graph
+
+        return cytoscape_graph(dsk, filename=filename, **kwargs)
+    elif engine is None:
+        raise RuntimeError(
+            "No visualization engine detected, please install graphviz or ipycytoscape"
+        )
+    else:
+        raise ValueError(f"Visualization engine {engine} not recognized")
 
 
 def persist(*args, traverse=True, optimize_graph=True, scheduler=None, **kwargs):
@@ -832,7 +961,9 @@ def persist(*args, traverse=True, optimize_graph=True, scheduler=None, **kwargs)
         keys.extend(a_keys)
         postpersists.append((rebuild, a_keys, state))
 
-    results = schedule(dsk, keys, **kwargs)
+    with shorten_traceback():
+        results = schedule(dsk, keys, **kwargs)
+
     d = dict(zip(keys, results))
     results2 = [r({k: d[k] for k in ks}, *s) for r, ks, s in postpersists]
     return repack(results2)
@@ -843,6 +974,18 @@ def persist(*args, traverse=True, optimize_graph=True, scheduler=None, **kwargs)
 ############
 
 
+class _HashFactory(Protocol):
+    def __call__(
+        self, string: ReadableBuffer = b"", *, usedforsecurity: bool = True
+    ) -> hashlib._Hash:
+        ...
+
+
+# Pass `usedforsecurity=False` to support FIPS builds of Python
+def _md5(x: ReadableBuffer, _hashlib_md5: _HashFactory = hashlib.md5) -> hashlib._Hash:
+    return _hashlib_md5(x, usedforsecurity=False)
+
+
 def tokenize(*args, **kwargs):
     """Deterministic token
 
@@ -852,9 +995,10 @@ def tokenize(*args, **kwargs):
     >>> tokenize('Hello') == tokenize('Hello')
     True
     """
+    hasher = _md5(str(tuple(map(normalize_token, args))).encode())
     if kwargs:
-        args = args + (kwargs,)
-    return md5(str(tuple(map(normalize_token, args))).encode()).hexdigest()
+        hasher.update(str(normalize_token(kwargs)).encode())
+    return hasher.hexdigest()
 
 
 normalize_token = Dispatch()
@@ -870,6 +1014,10 @@ normalize_token.register(
         complex,
         type(Ellipsis),
         datetime.date,
+        datetime.time,
+        datetime.datetime,
+        datetime.timedelta,
+        pathlib.PurePath,
     ),
     identity,
 )
@@ -891,7 +1039,7 @@ def normalize_set(s):
 
 
 def _normalize_seq_func(seq):
-    # Defined outside normalize_seq to avoid unneccessary redefinitions and
+    # Defined outside normalize_seq to avoid unnecessary redefinitions and
     # therefore improving computation times.
     try:
         return list(map(normalize_token, seq))
@@ -921,6 +1069,11 @@ def normalize_range(r):
     return list(map(normalize_token, [r.start, r.stop, r.step]))
 
 
+@normalize_token.register(Enum)
+def normalize_enum(e):
+    return type(e).__name__, e.name, e.value
+
+
 @normalize_token.register(object)
 def normalize_object(o):
     method = getattr(o, "__dask_tokenize__", None)
@@ -930,6 +1083,9 @@ def normalize_object(o):
     if callable(o):
         return normalize_function(o)
 
+    if dataclasses.is_dataclass(o):
+        return normalize_dataclass(o)
+
     if not config.get("tokenize.ensure-deterministic"):
         return uuid.uuid4().hex
 
@@ -940,11 +1096,11 @@ def normalize_object(o):
     )
 
 
-function_cache: dict[Callable, Callable] = {}
+function_cache: dict[Callable, Callable | tuple | str | bytes] = {}
 function_cache_lock = threading.Lock()
 
 
-def normalize_function(func: Callable) -> Callable:
+def normalize_function(func: Callable) -> Callable | tuple | str | bytes:
     try:
         return function_cache[func]
     except KeyError:
@@ -960,7 +1116,7 @@ def normalize_function(func: Callable) -> Callable:
         return _normalize_function(func)
 
 
-def _normalize_function(func: Callable) -> Callable:
+def _normalize_function(func: Callable) -> tuple | str | bytes:
     if isinstance(func, Compose):
         first = getattr(func, "first", None)
         funcs = reversed((first,) + func.funcs) if first else func.funcs
@@ -976,25 +1132,40 @@ def _normalize_function(func: Callable) -> Callable:
         return (normalize_function(func.func), args, kws)
     else:
         try:
-            result = pickle.dumps(func, protocol=0)
+            result = pickle.dumps(func, protocol=4)
             if b"__main__" not in result:  # abort on dynamic functions
                 return result
         except Exception:
             pass
-        try:
-            import cloudpickle
+        if not config.get("tokenize.ensure-deterministic"):
+            try:
+                import cloudpickle
 
-            return cloudpickle.dumps(func, protocol=0)
-        except Exception:
-            return str(func)
+                return cloudpickle.dumps(func, protocol=4)
+            except Exception:
+                return str(func)
+        else:
+            raise RuntimeError(
+                f"Function {str(func)} may not be deterministically hashed by "
+                "cloudpickle. See: https://github.com/cloudpipe/cloudpickle/issues/385 "
+                "for more information."
+            )
+
+
+def normalize_dataclass(obj):
+    fields = [
+        (field.name, getattr(obj, field.name)) for field in dataclasses.fields(obj)
+    ]
+    return (
+        normalize_function(type(obj)),
+        _normalize_seq_func(fields),
+    )
 
 
 @normalize_token.register_lazy("pandas")
 def register_pandas():
     import pandas as pd
 
-    PANDAS_GT_130 = parse_version(pd.__version__) >= parse_version("1.3.0")
-
     @normalize_token.register(pd.Index)
     def normalize_index(ind):
         values = ind.array
@@ -1038,16 +1209,8 @@ def register_pandas():
 
     @normalize_token.register(pd.DataFrame)
     def normalize_dataframe(df):
-        mgr = df._data
-
-        if PANDAS_GT_130:
-            # for compat with ArrayManager, pandas 1.3.0 introduced a `.arrays`
-            # attribute that returns the column arrays/block arrays for both
-            # BlockManager and ArrayManager
-            data = list(mgr.arrays)
-        else:
-            data = [block.values for block in mgr.blocks]
-        data.extend([df.columns, df.index])
+        mgr = df._mgr
+        data = list(mgr.arrays) + [df.columns, df.index]
         return list(map(normalize_token, data))
 
     @normalize_token.register(pd.api.extensions.ExtensionArray)
@@ -1065,6 +1228,14 @@ def register_pandas():
     def normalize_period_dtype(dtype):
         return normalize_token(dtype.name)
 
+    @normalize_token.register(type(pd.NA))
+    def normalize_na(na):
+        return pd.NA
+
+    @normalize_token.register(pd.offsets.BaseOffset)
+    def normalize_offset(offset):
+        return offset.freqstr
+
 
 @normalize_token.register_lazy("numpy")
 def register_numpy():
@@ -1083,8 +1254,8 @@ def register_numpy():
                 offset = 0  # root memmap's have mmap object as base
             if hasattr(
                 x, "offset"
-            ):  # offset numpy used while opening, and not the offset to the beginning of the file
-                offset += getattr(x, "offset")
+            ):  # offset numpy used while opening, and not the offset to the beginning of file
+                offset += x.offset
             return (
                 x.filename,
                 os.path.getmtime(x.filename),
@@ -1142,6 +1313,10 @@ def register_numpy():
         except AttributeError:
             return normalize_function(x)
 
+    @normalize_token.register(np.random.BitGenerator)
+    def normalize_bit_generator(bg):
+        return normalize_token(bg.state)
+
 
 @normalize_token.register_lazy("scipy")
 def register_scipy():
@@ -1187,19 +1362,24 @@ def _colorize(t):
     return "#" + h
 
 
-named_schedulers = {
+named_schedulers: dict[str, SchedulerGetCallable] = {
     "sync": local.get_sync,
     "synchronous": local.get_sync,
     "single-threaded": local.get_sync,
-    "threads": threaded.get,
-    "threading": threaded.get,
 }
 
-try:
+if not EMSCRIPTEN:
+    from dask import threaded
+
+    named_schedulers.update(
+        {
+            "threads": threaded.get,
+            "threading": threaded.get,
+        }
+    )
+
     from dask import multiprocessing as dask_multiprocessing
-except ImportError:
-    pass
-else:
+
     named_schedulers.update(
         {
             "processes": dask_multiprocessing.get,
@@ -1235,7 +1415,8 @@ def get_scheduler(get=None, scheduler=None, collections=None, cls=None):
 
     1.  Passing in scheduler= parameters
     2.  Passing these into global configuration
-    3.  Using defaults of a dask collection
+    3.  Using a dask.distributed default Client
+    4.  Using defaults of a dask collection
 
     This function centralizes the logic to determine the right scheduler to use
     from those many options
@@ -1250,9 +1431,26 @@ def get_scheduler(get=None, scheduler=None, collections=None, cls=None):
             return scheduler.get
         elif isinstance(scheduler, str):
             scheduler = scheduler.lower()
+
+            try:
+                from distributed import Client
+
+                Client.current(allow_global=True)
+                client_available = True
+            except (ImportError, ValueError):
+                client_available = False
             if scheduler in named_schedulers:
+                if client_available:
+                    warnings.warn(
+                        "Running on a single-machine scheduler when a distributed client "
+                        "is active might lead to unexpected results."
+                    )
                 return named_schedulers[scheduler]
             elif scheduler in ("dask.distributed", "distributed"):
+                if not client_available:
+                    raise RuntimeError(
+                        f"Requested {scheduler} scheduler but no Client active."
+                    )
                 from distributed.worker import get_client
 
                 return get_client().get
@@ -1280,10 +1478,12 @@ def get_scheduler(get=None, scheduler=None, collections=None, cls=None):
     if config.get("get", None):
         raise ValueError(get_err_msg)
 
-    if getattr(thread_state, "key", False):
-        from distributed.worker import get_worker
+    try:
+        from distributed import get_client
 
-        return get_worker().client.get
+        return get_client().get
+    except (ImportError, ValueError):
+        pass
 
     if cls is not None:
         return cls.__dask_scheduler__
@@ -1340,7 +1540,7 @@ def get_collection_names(collection) -> set[str]:
     return {get_name_from_key(k) for k in flatten(collection.__dask_keys__())}
 
 
-def get_name_from_key(key) -> str:
+def get_name_from_key(key: Key) -> str:
     """Given a dask collection's key, extract the collection name.
 
     Parameters
@@ -1360,10 +1560,13 @@ def get_name_from_key(key) -> str:
         return key[0]
     if isinstance(key, str):
         return key
-    raise TypeError(f"Expected str or tuple[str, Hashable, ...]; got {key}")
+    raise TypeError(f"Expected str or a tuple starting with str; got {key!r}")
 
 
-def replace_name_in_key(key, rename: Mapping[str, str]):
+KeyOrStrT = TypeVar("KeyOrStrT", Key, str)
+
+
+def replace_name_in_key(key: KeyOrStrT, rename: Mapping[str, str]) -> KeyOrStrT:
     """Given a dask collection's key, replace the collection name with a new one.
 
     Parameters
@@ -1388,27 +1591,25 @@ def replace_name_in_key(key, rename: Mapping[str, str]):
         return (rename.get(key[0], key[0]),) + key[1:]
     if isinstance(key, str):
         return rename.get(key, key)
-    raise TypeError(f"Expected str or tuple[str, Hashable, ...]; got {key}")
+    raise TypeError(f"Expected str or a tuple starting with str; got {key!r}")
 
 
-def clone_key(key, seed):
+def clone_key(key: KeyOrStrT, seed: Hashable) -> KeyOrStrT:
     """Clone a key from a Dask collection, producing a new key with the same prefix and
-    indices and a token which a deterministic function of the previous token and seed.
+    indices and a token which is a deterministic function of the previous key and seed.
 
     Examples
     --------
-    >>> clone_key("inc-cbb1eca3bafafbb3e8b2419c4eebb387", 123)  # doctest: +SKIP
-    'inc-1d291de52f5045f8a969743daea271fd'
-    >>> clone_key(("sum-cbb1eca3bafafbb3e8b2419c4eebb387", 4, 3), 123)  # doctest: +SKIP
-    ('sum-f0962cc58ef4415689a86cc1d4cc1723', 4, 3)
+    >>> clone_key("x", 123)
+    'x-dc2b8d1c184c72c19faa81c797f8c6b0'
+    >>> clone_key("inc-cbb1eca3bafafbb3e8b2419c4eebb387", 123)
+    'inc-f81b5a88038a2132882aa29a9fcfec06'
+    >>> clone_key(("sum-cbb1eca3bafafbb3e8b2419c4eebb387", 4, 3), 123)
+    ('sum-fd6be9e9fe07fc232ad576fa997255e8', 4, 3)
     """
     if isinstance(key, tuple) and key and isinstance(key[0], str):
         return (clone_key(key[0], seed),) + key[1:]
     if isinstance(key, str):
         prefix = key_split(key)
-        token = key[len(prefix) + 1 :]
-        if token:
-            return prefix + "-" + tokenize(token, seed)
-        else:
-            return tokenize(key, seed)
-    raise TypeError(f"Expected str or tuple[str, Hashable, ...]; got {key}")
+        return prefix + "-" + tokenize(key, seed)
+    raise TypeError(f"Expected str or a tuple starting with str; got {key!r}")
