@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import contextlib
 
 import numpy as np
@@ -5,7 +7,9 @@ import pytest
 
 pd = pytest.importorskip("pandas")
 import dask.dataframe as dd
-from dask.dataframe.utils import assert_eq
+from dask.dataframe._compat import PANDAS_GE_140, PANDAS_GE_210
+from dask.dataframe._pyarrow import to_pyarrow_string
+from dask.dataframe.utils import assert_eq, pyarrow_strings_enabled
 
 
 @contextlib.contextmanager
@@ -98,16 +102,22 @@ def test_dt_accessor(df_ddf):
     # see https://github.com/pydata/pandas/issues/10712
     assert_eq(ddf.dt_col.dt.date, df.dt_col.dt.date, check_names=False)
 
+    warning = FutureWarning if PANDAS_GE_210 else None
     # to_pydatetime returns a numpy array in pandas, but a Series in dask
-    assert_eq(
-        ddf.dt_col.dt.to_pydatetime(),
-        pd.Series(df.dt_col.dt.to_pydatetime(), index=df.index, dtype=object),
-    )
+    # pandas will start returning a Series with 3.0 as well
+    with pytest.warns(warning, match="will return a Series"):
+        ddf_result = ddf.dt_col.dt.to_pydatetime()
+    with pytest.warns(warning, match="will return a Series"):
+        pd_result = pd.Series(
+            df.dt_col.dt.to_pydatetime(), index=df.index, dtype=object
+        )
+    assert_eq(ddf_result, pd_result)
 
     assert set(ddf.dt_col.dt.date.dask) == set(ddf.dt_col.dt.date.dask)
-    assert set(ddf.dt_col.dt.to_pydatetime().dask) == set(
-        ddf.dt_col.dt.to_pydatetime().dask
-    )
+    with pytest.warns(warning, match="will return a Series"):
+        assert set(ddf.dt_col.dt.to_pydatetime().dask) == set(
+            ddf.dt_col.dt.to_pydatetime().dask
+        )
 
 
 def test_dt_accessor_not_available(df_ddf):
@@ -142,15 +152,30 @@ def test_str_accessor(df_ddf):
     assert set(ddf.index.str.upper().dask) == set(ddf.index.str.upper().dask)
 
     # make sure to pass through args & kwargs
-    assert_eq(ddf.str_col.str.contains("a"), df.str_col.str.contains("a"))
+    # NOTE: when using pyarrow strings, `.str.contains(...)` will return a result
+    # with `boolean` dtype, while using object strings returns a `bool`. We cast
+    # the pandas DataFrame here to ensure pandas and Dask return the same dtype.
+    ctx = contextlib.nullcontext()
+    if pyarrow_strings_enabled():
+        df.str_col = to_pyarrow_string(df.str_col)
+        if not PANDAS_GE_210:
+            ctx = pytest.warns(
+                pd.errors.PerformanceWarning, match="Falling back on a non-pyarrow"
+            )
+    assert_eq(
+        ddf.str_col.str.contains("a"),
+        df.str_col.str.contains("a"),
+    )
     assert_eq(ddf.string_col.str.contains("a"), df.string_col.str.contains("a"))
     assert set(ddf.str_col.str.contains("a").dask) == set(
         ddf.str_col.str.contains("a").dask
     )
 
+    with ctx:
+        expected = df.str_col.str.contains("d", case=False)
     assert_eq(
         ddf.str_col.str.contains("d", case=False),
-        df.str_col.str.contains("d", case=False),
+        expected,
     )
     assert set(ddf.str_col.str.contains("d", case=False).dask) == set(
         ddf.str_col.str.contains("d", case=False).dask
@@ -158,7 +183,8 @@ def test_str_accessor(df_ddf):
 
     for na in [True, False]:
         assert_eq(
-            ddf.str_col.str.contains("a", na=na), df.str_col.str.contains("a", na=na)
+            ddf.str_col.str.contains("a", na=na),
+            df.str_col.str.contains("a", na=na),
         )
         assert set(ddf.str_col.str.contains("a", na=na).dask) == set(
             ddf.str_col.str.contains("a", na=na).dask
@@ -198,6 +224,22 @@ def test_str_accessor_extractall(df_ddf):
     )
 
 
+@pytest.mark.skipif(not PANDAS_GE_140, reason="requires pandas >= 1.4.0")
+@pytest.mark.parametrize("method", ["removeprefix", "removesuffix"])
+def test_str_accessor_removeprefix_removesuffix(df_ddf, method):
+    df, ddf = df_ddf
+    prefix = df.str_col.iloc[0][:2]
+    suffix = df.str_col.iloc[0][-2:]
+    missing = "definitely a missing prefix/suffix"
+
+    def call(df, arg):
+        return getattr(df.str_col.str, method)(arg)
+
+    assert_eq(call(ddf, prefix), call(df, prefix))
+    assert_eq(call(ddf, suffix), call(df, suffix))
+    assert_eq(call(ddf, missing), call(df, missing))
+
+
 def test_str_accessor_cat(df_ddf):
     df, ddf = df_ddf
     sol = df.str_col.str.cat(df.str_col.str.upper(), sep=":")
@@ -223,27 +265,35 @@ def test_str_accessor_cat_none():
     assert_eq(ds.str.cat(sep="_", na_rep="-"), s.str.cat(sep="_", na_rep="-"))
 
 
-def test_str_accessor_noexpand():
+@pytest.mark.parametrize("method", ["split", "rsplit"])
+def test_str_accessor_split_noexpand(method):
+    def call(obj, *args, **kwargs):
+        return getattr(obj.str, method)(*args, **kwargs)
+
     s = pd.Series(["a b c d", "aa bb cc dd", "aaa bbb ccc dddd"], name="foo")
     ds = dd.from_pandas(s, npartitions=2)
 
     for n in [1, 2, 3]:
-        assert_eq(s.str.split(n=n, expand=False), ds.str.split(n=n, expand=False))
+        assert_eq(call(s, n=n, expand=False), call(ds, n=n, expand=False))
 
-    assert ds.str.split(n=1, expand=False).name == "foo"
+    assert call(ds, n=1, expand=False).name == "foo"
 
 
-def test_str_accessor_expand():
+@pytest.mark.parametrize("method", ["split", "rsplit"])
+def test_str_accessor_split_expand(method):
+    def call(obj, *args, **kwargs):
+        return getattr(obj.str, method)(*args, **kwargs)
+
     s = pd.Series(
         ["a b c d", "aa bb cc dd", "aaa bbb ccc dddd"], index=["row1", "row2", "row3"]
     )
     ds = dd.from_pandas(s, npartitions=2)
 
     for n in [1, 2, 3]:
-        assert_eq(s.str.split(n=n, expand=True), ds.str.split(n=n, expand=True))
+        assert_eq(call(s, n=n, expand=True), call(ds, n=n, expand=True))
 
     with pytest.raises(NotImplementedError) as info:
-        ds.str.split(expand=True)
+        call(ds, expand=True)
 
     assert "n=" in str(info.value)
 
@@ -252,13 +302,12 @@ def test_str_accessor_expand():
 
     for n in [1, 2, 3]:
         assert_eq(
-            s.str.split(pat=",", n=n, expand=True),
-            ds.str.split(pat=",", n=n, expand=True),
+            call(s, pat=",", n=n, expand=True), call(ds, pat=",", n=n, expand=True)
         )
 
 
 @pytest.mark.xfail(reason="Need to pad columns")
-def test_str_accessor_expand_more_columns():
+def test_str_accessor_split_expand_more_columns():
     s = pd.Series(["a b c d", "aa", "aaa bbb ccc dddd"])
     ds = dd.from_pandas(s, npartitions=2)
 
@@ -270,6 +319,17 @@ def test_str_accessor_expand_more_columns():
     ds.str.split(n=10, expand=True).compute()
 
 
+@pytest.mark.parametrize("index", [None, [0]], ids=["range_index", "other index"])
+def test_str_split_no_warning(index):
+    df = pd.DataFrame({"a": ["a\nb"]}, index=index)
+    ddf = dd.from_pandas(df, npartitions=1)
+
+    pd_a = df["a"].str.split("\n", n=1, expand=True)
+    dd_a = ddf["a"].str.split("\n", n=1, expand=True)
+
+    assert_eq(dd_a, pd_a)
+
+
 def test_string_nullable_types(df_ddf):
     df, ddf = df_ddf
     assert_eq(ddf.string_col.str.count("A"), df.string_col.str.count("A"))
