@@ -1,3 +1,7 @@
+from __future__ import annotations
+
+import contextlib
+
 import numpy as np
 import pandas as pd
 import pytest
@@ -5,9 +9,9 @@ import pytest
 import dask
 import dask.dataframe as dd
 from dask.base import tokenize
-from dask.dataframe._compat import PANDAS_GT_110, PANDAS_GT_120, tm
+from dask.dataframe._compat import PANDAS_GE_210, PANDAS_GE_220, IndexingError, tm
 from dask.dataframe.indexing import _coerce_loc_index
-from dask.dataframe.utils import assert_eq, make_meta
+from dask.dataframe.utils import assert_eq, make_meta, pyarrow_strings_enabled
 
 dsk = {
     ("x", 0): pd.DataFrame({"a": [1, 2, 3], "b": [4, 5, 6]}, index=[0, 1, 3]),
@@ -17,11 +21,12 @@ dsk = {
 meta = make_meta(
     {"a": "i8", "b": "i8"}, index=pd.Index([], "i8"), parent_meta=pd.DataFrame()
 )
-d = dd.DataFrame(dsk, "x", meta, [0, 5, 9, 9])
-full = d.compute()
-CHECK_FREQ = {}
-if PANDAS_GT_110:
-    CHECK_FREQ["check_freq"] = False
+if not dd._dask_expr_enabled():
+    d = dd.DataFrame(dsk, "x", meta, [0, 5, 9, 9])
+    full = d.compute()
+else:
+    d = dd.repartition(pd.concat(dsk.values()), divisions=[0, 5, 9, 9])
+    full = d.compute()
 
 
 def test_loc():
@@ -156,8 +161,14 @@ def test_loc_with_non_boolean_series():
 
     assert_eq(ddf.loc[s], df.loc[s])
 
-    with pytest.raises(KeyError, match=msg):
-        ddf.loc[ds.values]
+    ctx = contextlib.nullcontext()
+    if pyarrow_strings_enabled():
+        ctx = pytest.warns(
+            UserWarning, match="converting pandas extension dtypes to arrays"
+        )
+    with ctx:
+        with pytest.raises(KeyError, match=msg):
+            ddf.loc[ds.values]
 
     assert_eq(ddf.loc[s.values], df.loc[s])
 
@@ -188,17 +199,17 @@ def test_loc2d():
     assert_eq(d.loc[3:, ["a"]], full.loc[3:, ["a"]])
 
     # 3d
-    with pytest.raises(pd.core.indexing.IndexingError):
+    with pytest.raises(IndexingError):
         d.loc[3, 3, 3]
 
     # Series should raise
-    with pytest.raises(pd.core.indexing.IndexingError):
+    with pytest.raises(IndexingError):
         d.a.loc[3, 3]
 
-    with pytest.raises(pd.core.indexing.IndexingError):
+    with pytest.raises(IndexingError):
         d.a.loc[3:, 3]
 
-    with pytest.raises(pd.core.indexing.IndexingError):
+    with pytest.raises(IndexingError):
         d.a.loc[d.a % 2 == 0, 3]
 
 
@@ -328,9 +339,15 @@ def test_getitem_integer_slice():
     df = pd.DataFrame({"A": range(6)}, index=[1.0, 2.0, 3.0, 5.0, 10.0, 11.0])
     ddf = dd.from_pandas(df, 2)
     # except for float dtype indexes
-    assert_eq(ddf[2:8], df[2:8])
-    assert_eq(ddf[2:], df[2:])
-    assert_eq(ddf[:8], df[:8])
+    ctx = contextlib.nullcontext()
+    if PANDAS_GE_210:
+        ctx = pytest.warns(FutureWarning, match="float-dtype index")
+    with ctx:
+        assert_eq(ddf[2:8], df[2:8])
+    with ctx:
+        assert_eq(ddf[2:], df[2:])
+    with ctx:
+        assert_eq(ddf[:8], df[:8])
 
 
 def test_loc_on_numpy_datetimes():
@@ -372,11 +389,13 @@ def test_coerce_loc_index():
         assert isinstance(_coerce_loc_index([t("2014")], "2014"), t)
 
 
-def test_loc_timestamp_str():
+ME = "ME" if PANDAS_GE_220 else "M"
 
+
+def test_loc_timestamp_str():
     df = pd.DataFrame(
         {"A": np.random.randn(100), "B": np.random.randn(100)},
-        index=pd.date_range("2011-01-01", freq="H", periods=100),
+        index=pd.date_range("2011-01-01", freq="h", periods=100),
     )
     ddf = dd.from_pandas(df, 10)
 
@@ -387,37 +406,37 @@ def test_loc_timestamp_str():
     assert_eq(
         df.loc["2011-01-02 10:00"].to_frame().T,
         ddf.loc["2011-01-02 10:00"],
-        **CHECK_FREQ,
+        check_freq=False,
     )
 
     # series
-    assert_eq(df.A.loc["2011-01-02"], ddf.A.loc["2011-01-02"], **CHECK_FREQ)
+    assert_eq(df.A.loc["2011-01-02"], ddf.A.loc["2011-01-02"], check_freq=False)
     assert_eq(
         df.A.loc["2011-01-02":"2011-01-10"],
         ddf.A.loc["2011-01-02":"2011-01-10"],
-        **CHECK_FREQ,
+        check_freq=False,
     )
 
     # slice with timestamp (dask result must be DataFrame)
     assert_eq(
         df.loc[pd.Timestamp("2011-01-02")].to_frame().T,
         ddf.loc[pd.Timestamp("2011-01-02")],
-        **CHECK_FREQ,
+        check_freq=False,
     )
     assert_eq(
         df.loc[pd.Timestamp("2011-01-02") : pd.Timestamp("2011-01-10")],
         ddf.loc[pd.Timestamp("2011-01-02") : pd.Timestamp("2011-01-10")],
-        **CHECK_FREQ,
+        check_freq=False,
     )
     assert_eq(
         df.loc[pd.Timestamp("2011-01-02 10:00")].to_frame().T,
         ddf.loc[pd.Timestamp("2011-01-02 10:00")],
-        **CHECK_FREQ,
+        check_freq=False,
     )
 
     df = pd.DataFrame(
         {"A": np.random.randn(100), "B": np.random.randn(100)},
-        index=pd.date_range("2011-01-01", freq="M", periods=100),
+        index=pd.date_range("2011-01-01", freq=ME, periods=100),
     )
     ddf = dd.from_pandas(df, 50)
     assert_eq(df.loc["2011-01"], ddf.loc["2011-01"])
@@ -435,19 +454,13 @@ def test_loc_timestamp_str():
 
 
 def test_getitem_timestamp_str():
-
     df = pd.DataFrame(
         {"A": np.random.randn(100), "B": np.random.randn(100)},
-        index=pd.date_range("2011-01-01", freq="H", periods=100),
+        index=pd.date_range("2011-01-01", freq="h", periods=100),
     )
     ddf = dd.from_pandas(df, 10)
 
-    if PANDAS_GT_120:
-        with pytest.warns(
-            FutureWarning, match="Indexing a DataFrame with a datetimelike"
-        ):
-            assert_eq(df.loc["2011-01-02"], ddf["2011-01-02"])
-    else:
+    with pytest.warns(FutureWarning, match="Indexing a DataFrame with a datetimelike"):
         assert_eq(df.loc["2011-01-02"], ddf["2011-01-02"])
     assert_eq(df["2011-01-02":"2011-01-10"], ddf["2011-01-02":"2011-01-10"])
 
@@ -463,16 +476,13 @@ def test_getitem_timestamp_str():
     assert_eq(df["2011":"2015"], ddf["2011":"2015"])
 
 
-@pytest.mark.xfail(
-    not PANDAS_GT_110, reason=".loc partial index with PeriodIndex not yet supported"
-)
 def test_loc_period_str():
     # .loc with PeriodIndex doesn't support partial string indexing
     # https://github.com/pydata/pandas/issues/13429
     # -> this started working in pandas 1.1
     df = pd.DataFrame(
         {"A": np.random.randn(100), "B": np.random.randn(100)},
-        index=pd.period_range("2011-01-01", freq="H", periods=100),
+        index=pd.period_range("2011-01-01", freq="h", periods=100),
     )
     ddf = dd.from_pandas(df, 10)
 
@@ -494,21 +504,15 @@ def test_loc_period_str():
 
 
 def test_getitem_period_str():
-
     df = pd.DataFrame(
         {"A": np.random.randn(100), "B": np.random.randn(100)},
-        index=pd.period_range("2011-01-01", freq="H", periods=100),
+        index=pd.period_range("2011-01-01", freq="h", periods=100),
     )
     ddf = dd.from_pandas(df, 10)
 
     # partial string slice
-    if PANDAS_GT_120:
-        with pytest.warns(
-            FutureWarning, match="Indexing a DataFrame with a datetimelike"
-        ):
-            assert_eq(df.loc["2011-01-02"], ddf["2011-01-02"])
-    else:
-        assert_eq(df["2011-01-02"], ddf["2011-01-02"])
+    with pytest.warns(FutureWarning, match="Indexing a DataFrame with a datetimelike"):
+        assert_eq(df.loc["2011-01-02"], ddf["2011-01-02"])
     assert_eq(df["2011-01-02":"2011-01-10"], ddf["2011-01-02":"2011-01-10"])
     # same reso, dask result is always DataFrame
 
@@ -518,21 +522,11 @@ def test_getitem_period_str():
     )
     ddf = dd.from_pandas(df, 50)
 
-    if PANDAS_GT_120:
-        with pytest.warns(
-            FutureWarning, match="Indexing a DataFrame with a datetimelike"
-        ):
-            assert_eq(df.loc["2011-01"], ddf["2011-01"])
-    else:
-        assert_eq(df["2011-01"], ddf["2011-01"])
+    with pytest.warns(FutureWarning, match="Indexing a DataFrame with a datetimelike"):
+        assert_eq(df.loc["2011-01"], ddf["2011-01"])
 
-    if PANDAS_GT_120:
-        with pytest.warns(
-            FutureWarning, match="Indexing a DataFrame with a datetimelike"
-        ):
-            assert_eq(df.loc["2011"], ddf["2011"])
-    else:
-        assert_eq(df["2011"], ddf["2011"])
+    with pytest.warns(FutureWarning, match="Indexing a DataFrame with a datetimelike"):
+        assert_eq(df.loc["2011"], ddf["2011"])
 
     assert_eq(df["2011-01":"2012-05"], ddf["2011-01":"2012-05"])
     assert_eq(df["2011":"2015"], ddf["2011":"2015"])
@@ -541,7 +535,7 @@ def test_getitem_period_str():
 @pytest.mark.parametrize(
     "index",
     [
-        pd.date_range("2011-01-01", freq="H", periods=100),  # time index
+        pd.date_range("2011-01-01", freq="h", periods=100),  # time index
         range(100),  # numerical index
     ],
 )
@@ -559,7 +553,7 @@ def test_to_series(index):
 @pytest.mark.parametrize(
     "index",
     [
-        pd.date_range("2011-01-01", freq="H", periods=100),  # time index
+        pd.date_range("2011-01-01", freq="h", periods=100),  # time index
         range(100),  # numerical index
     ],
 )
