@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import pandas as pd
 
 import dask
@@ -32,7 +34,7 @@ def test_optimize_blockwise():
     df = pd.DataFrame({"x": range(10), "y": range(10)})
     ddf = dd.from_pandas(df, npartitions=2)
 
-    for i in range(10):
+    for _ in range(10):
         ddf["x"] = ddf.x + 1 + ddf.y
 
     graph = optimize_blockwise(ddf.dask)
