@@ -1,10 +1,13 @@
-import importlib.metadata
+from __future__ import annotations
+
 import os
 
+import importlib_metadata
 import pytest
 from packaging.version import Version
 
 
+@pytest.mark.xfail(reason="https://github.com/dask/dask/issues/9735", strict=False)
 @pytest.mark.skipif(
     not os.environ.get("UPSTREAM_DEV", False),
     reason="Only check for dev packages in `upstream` CI build",
@@ -26,7 +29,7 @@ def test_upstream_packages_installed():
         "numpy",
         "pandas",
         # "partd",
-        # "pyarrow",
+        "pyarrow",
         # "s3fs",
         "scipy",
         # "sparse",
@@ -34,5 +37,5 @@ def test_upstream_packages_installed():
         # "zict",
     ]
     for package in packages:
-        v = Version(importlib.metadata.version(package))
+        v = Version(importlib_metadata.version(package))
         assert v.is_prerelease or v.local is not None, (package, str(v))
