@@ -1,6 +1,10 @@
+from __future__ import annotations
+
 import os
+import threading
 import xml.etree.ElementTree
 from collections.abc import Set
+from concurrent.futures import ThreadPoolExecutor
 
 import pytest
 
@@ -49,20 +53,6 @@ def test_keys_values_items_to_dict_methods():
     assert hg.to_dict() == dict(hg)
 
 
-def test_keyset_deprecated():
-    a = {"x": 1, "y": (inc, "x")}
-    hg = HighLevelGraph({"a": a}, {"a": set()})
-    with pytest.warns(FutureWarning, match="HighLevelGraph.keys"):
-        assert hg.keyset() == hg.keys()
-
-
-def test_dicts_deprecated():
-    a = {"x": 1, "y": (inc, "x")}
-    hg = HighLevelGraph({"a": a}, {"a": set()})
-    with pytest.warns(FutureWarning, match="HighLevelGraph.layers"):
-        assert hg.dicts == hg.layers
-
-
 def test_getitem():
     hg = HighLevelGraph(
         {"a": {"a": 1, ("a", 0): 2, "b": 3}, "b": {"c": 4}}, {"a": set(), "b": set()}
@@ -171,7 +161,7 @@ def test_single_annotation(annotation):
 
     alayer = A.__dask_graph__().layers[A.name]
     assert alayer.annotations == annotation
-    assert dask.config.get("annotations", None) is None
+    assert not dask.get_annotations()
 
 
 def test_multiple_annotations():
@@ -184,7 +174,7 @@ def test_multiple_annotations():
 
     C = B + 1
 
-    assert dask.config.get("annotations", None) is None
+    assert not dask.get_annotations()
 
     alayer = A.__dask_graph__().layers[A.name]
     blayer = B.__dask_graph__().layers[B.name]
@@ -194,14 +184,14 @@ def test_multiple_annotations():
     assert clayer.annotations is None
 
 
-def test_annotation_pack_unpack():
-    layer = MaterializedLayer({"n": 42}, annotations={"workers": ("alice",)})
-    packed_anno = layer.__dask_distributed_annotations_pack__()
-    annotations = {}
-    Layer.__dask_distributed_annotations_unpack__(
-        annotations, packed_anno, layer.keys()
-    )
-    assert annotations == {"workers": {"n": ("alice",)}}
+def test_annotation_cleared_on_error():
+    with dask.annotate(x=1):
+        with pytest.raises(ZeroDivisionError):
+            with dask.annotate(x=2):
+                assert dask.get_annotations() == {"x": 2}
+                1 / 0
+        assert dask.get_annotations() == {"x": 1}
+    assert not dask.get_annotations()
 
 
 def test_materializedlayer_cull_preserves_annotations():
@@ -215,6 +205,26 @@ def test_materializedlayer_cull_preserves_annotations():
     assert culled_layer.annotations == {"foo": "bar"}
 
 
+def test_annotations_leak():
+    """Annotations shouldn't leak between threads.
+    See https://github.com/dask/dask/issues/10340."""
+    b1 = threading.Barrier(2)
+    b2 = threading.Barrier(2)
+
+    def f(n):
+        with dask.annotate(foo=n):
+            b1.wait()
+            out = dask.get_annotations()
+            b2.wait()
+            return out
+
+    with ThreadPoolExecutor(2) as ex:
+        f1 = ex.submit(f, 1)
+        f2 = ex.submit(f, 2)
+        result = [f1.result(), f2.result()]
+    assert result == [{"foo": 1}, {"foo": 2}]
+
+
 @pytest.mark.parametrize("flat", [True, False])
 def test_blockwise_cull(flat):
     da = pytest.importorskip("dask.array")
@@ -248,13 +258,6 @@ def test_blockwise_cull(flat):
         assert not layer.is_materialized()
 
 
-def test_highlevelgraph_dicts_deprecation():
-    with pytest.warns(FutureWarning):
-        layers = {"a": MaterializedLayer({"x": 1, "y": (inc, "x")})}
-        hg = HighLevelGraph(layers, {"a": set()})
-        assert hg.dicts == layers
-
-
 def test_len_does_not_materialize():
     a = {"x": 1}
     b = Blockwise(
