@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import pytest
 
 import dask
@@ -167,7 +169,7 @@ def test_ordering():
 
     get_sync(dsk, "y")
 
-    assert L == sorted(L)
+    assert L == sorted(L, reverse=True)
 
 
 def test_complex_ordering():
